using System;
using System.Web.UI;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Web.UI.WebControls;
using System.Text.RegularExpressions;

namespace PathTrackingControlLib
{
	/// <summary>
	/// PathTrackingControl
	/// Version 1.0
	/// Written By Ralph Varjabedian
	/// All rights reserved
	/// visit http://www.bytesurge.com to get the latest version
	/// </summary>
	[DefaultProperty("Text"), 
	ToolboxBitmap(typeof(Resources.NamespaceTyper), "icon.bmp"),
	ToolboxData("<{0}:PathTrackingControl runat=server></{0}:PathTrackingControl>")]
	public class PathTrackingControl : System.Web.UI.WebControls.WebControl
	{
		#region Supporting structures
		struct Node
		{
			public Node(string title, string path, string description, string icon)
			{
				this.title = title;
				this.path = path;
				this.description = description;
				this.icon = icon;
			}

			/// <summary>
			/// The title of the page
			/// </summary>
			public string title;

			/// <summary>
			/// The path of the page
			/// </summary>
			public string path;

			/// <summary>
			/// Option icon to display near the hyperlink of the page
			/// </summary>
			public string icon;

			/// <summary>
			/// Page description, to be used as a tooltip
			/// </summary>
			public string description;
		};
		#endregion

		#region LinkCSSClass
		private string linkCSSClass = "";
		[Bindable(true), 
		Description("A CSS Class used for the link"),
		Category("Appearance"), 
		DefaultValue("")]
		public string LinkCSSClass 
		{
			get
			{
				return linkCSSClass;
			}

			set
			{
				linkCSSClass = value;
			}
		}
		#endregion

		#region TableCSSClass
		private string tableCSSClass = "";
		[Bindable(true), 
		Description("A CSS Class used for the table that embeds the hyperlinks"),
		Category("Appearance"), 
		DefaultValue("")]
		public string TableCSSClass
		{
			get
			{
				return tableCSSClass;
			}

			set
			{
				tableCSSClass = value;
			}
		}
		#endregion

		#region TableRowCSSClass
		private string tableRowCSSClass = "";
		[Bindable(true), 
		Description("A CSS Class used for the row in the table that embeds the hyperlinks"),
		Category("Appearance"), 
		DefaultValue("")]
		public string TableRowCSSClass
		{
			get
			{
				return tableRowCSSClass;
			}

			set
			{
				tableRowCSSClass = value;
			}
		}
		#endregion

		#region PreItemSeparatorInLink
		private string preItemSeparatorInLink = " > ";
		[Bindable(true), 
		Description("HTML code to be put infront of each link (inside the Anchor tag)"),
		Category("Behavior"), 
		DefaultValue(" > ")]
		public string PreItemSeparatorInLink
		{
			get
			{
				return preItemSeparatorInLink;
			}

			set
			{
				preItemSeparatorInLink = value;
			}
		}
		#endregion

		#region PreItemSeparatorOutLink
		private string preItemSeparatorOutLink = "";
		[Bindable(true), 
		Description("HTML code to be put infront of each link (outside the Anchor tag)"),
		Category("Behavior"), 
		DefaultValue("")]
		public string PreItemSeparatorOutLink
		{
			get
			{
				return preItemSeparatorOutLink;
			}

			set
			{
				preItemSeparatorOutLink = value;
			}
		}
		#endregion

		#region PostItemSeparatorInLink
		private string postItemSeparatorInLink = "";
		[Bindable(true), 
		Description("HTML code to be put after each link (inside the anchor tag)"),
		Category("Behavior"), 
		DefaultValue("")]
		public string PostItemSeparatorInLink
		{
			get
			{
				return postItemSeparatorInLink;
			}

			set
			{
				postItemSeparatorInLink = value;
			}
		}
		#endregion

		#region PostItemSeparatorOutLink
		private string postItemSeparatorOutLink = "&nbsp;";
		[Bindable(true), 
		Description("HTML code to be put after each link (outside the anchor tag)"),
		Category("Behavior"), 
		DefaultValue("&nbsp;")]
		public string PostItemSeparatorOutLink
		{
			get
			{
				return postItemSeparatorOutLink;
			}

			set
			{
				postItemSeparatorOutLink = value;
			}
		}
		#endregion

		/// <summary> 
		/// Render this control to the output parameter specified.
		/// </summary>
		/// <param name="output"> The HTML writer to write out to </param>
		protected override void RenderContents(HtmlTextWriter output)
		{
			// pre process the path array to cut it off if needed
			_PreProcessPathArray();

			// Process the current request/page
			_ProcessPath();

			// Print the current path
			_RenderCurrentPath(output);
		}

		#region Private methods
		private void _RenderCurrentPath(HtmlTextWriter output)
		{
			ArrayList pathArray = _GetPathArray();
			Page.Session[_GetPageSpecificSessionName()] = (int) pathArray.Count; // update max for current page, so as to extract this info when this page is a referral

			if (pathArray.Count == 0)
				return; // nothing yet

			output.AddAttribute(HtmlTextWriterAttribute.Align, "center");
			output.RenderBeginTag(HtmlTextWriterTag.Div);

			output.AddAttribute(HtmlTextWriterAttribute.Class, tableCSSClass);
			output.RenderBeginTag(HtmlTextWriterTag.Table);

			output.AddAttribute(HtmlTextWriterAttribute.Class, tableRowCSSClass);
			output.RenderBeginTag(HtmlTextWriterTag.Tr);

			output.RenderBeginTag(HtmlTextWriterTag.Td);

			Node node;
			string tooltip;
			for (int i = 0; i < pathArray.Count - 1; i++)
			{
				node = (Node) pathArray[i];
				tooltip = node.title;
				if (node.description != "")
					tooltip += (output.NewLine + node.description);
				// write the hyperlink
				output.AddAttribute(HtmlTextWriterAttribute.Class, linkCSSClass);
				output.AddAttribute(HtmlTextWriterAttribute.Title, "Go back to " + tooltip);
				output.AddAttribute(HtmlTextWriterAttribute.Href, node.path);
				output.Write(preItemSeparatorOutLink);
				output.RenderBeginTag(HtmlTextWriterTag.A);
				output.Write(preItemSeparatorInLink);
				if (node.icon != "")
				{
					output.AddAttribute(HtmlTextWriterAttribute.Border, "0");
					output.AddAttribute(HtmlTextWriterAttribute.Src, node.icon);
					output.RenderBeginTag(HtmlTextWriterTag.Img);
					output.Write("&nbsp;");
				}
				output.Write(node.title);
				output.Write(postItemSeparatorInLink);
				output.RenderEndTag();
				output.Write(postItemSeparatorOutLink);
			}

			node = (Node) pathArray[pathArray.Count - 1];
			tooltip = node.title;
			if (node.description != "")
				tooltip += (output.NewLine + node.description);

			output.AddAttribute(HtmlTextWriterAttribute.Class, linkCSSClass);
			output.AddAttribute(HtmlTextWriterAttribute.Title, "Refresh " + tooltip);
			output.AddAttribute(HtmlTextWriterAttribute.Href, node.path);
			output.Write(preItemSeparatorOutLink);
			output.RenderBeginTag(HtmlTextWriterTag.A);
			output.Write(preItemSeparatorInLink);
			if (node.icon != "")
			{
				output.AddAttribute(HtmlTextWriterAttribute.Border, "0");
				output.AddAttribute(HtmlTextWriterAttribute.Src, node.icon);
				output.RenderBeginTag(HtmlTextWriterTag.Img);
				output.Write("&nbsp;");
			}
			output.Write(node.title);
			output.Write(postItemSeparatorInLink);
			output.RenderEndTag();
			output.Write(postItemSeparatorOutLink);

			output.RenderEndTag(); // </td>
			output.RenderEndTag(); // </tr>
			output.RenderEndTag(); // </table>
			output.RenderEndTag(); // </div>
		}

		/// <summary>
		/// A method that gets the referrer last path state
		/// </summary>
		private void _PreProcessPathArray()
		{
			int x = -1;
			if (Page.Session[_GetRefererPageSpecificSessionName()] != null)
				x = (int) Page.Session[_GetRefererPageSpecificSessionName()];

			if (Page.Session["PathTrackingControl.PathArray"] != null)
			{
				ArrayList pathArray1 = _GetPathArray();
				if (pathArray1.Count > x && x != -1)
				{
					int extra = pathArray1.Count - x;
					pathArray1.RemoveRange(x, extra);
				}
			}
		}

		/// <summary>
		/// Process the current request/page
		/// </summary>
		private void _ProcessPath()
		{
			// get the current PathArray
			ArrayList pathArray = null;
			if (Page.Session["PathTrackingControl.PathArray"] != null)
				pathArray = _GetPathArray();

			if (pathArray == null)
			{
				pathArray = new ArrayList();
				Page.Session["PathTrackingControl.PathArray"] = pathArray;
			}

			// try to get the page name and description (tooltip) 
			// from parsing the page
			string title = "Unnamed page";
			string desc = "";
			string icon = "";
			if (this.Page.Controls[0] is System.Web.UI.LiteralControl)
			{
				System.Web.UI.LiteralControl lc = (System.Web.UI.LiteralControl) this.Page.Controls[0];

				Regex reg = new Regex("<meta name=\"description\" (C|c)ontent=\"(?<desc>.*)\">");
				Match match = reg.Match(lc.Text);
				desc = match.Groups["desc"].Value;

				reg = new Regex("<title>(?<desc>.*)</title>");
				match = reg.Match(lc.Text);
				title = match.Groups["desc"].Value;

				reg = new Regex("<meta name=\"icon\" (C|c)ontent=\"(?<icon>.*)\">");
				match = reg.Match(lc.Text);
				icon = match.Groups["icon"].Value;
			}

			// generate a new node according to the current page/request
			Node node = new Node(title, Page.Request.RawUrl, desc, icon);
			_ProcessNewPath(ref pathArray, ref node);
		}

		/// <summary>
		/// Gets the path array (one per application)
		/// </summary>
		/// <returns></returns>
		private ArrayList _GetPathArray()
		{
			return (ArrayList) Page.Session["PathTrackingControl.PathArray"];
		}

		/// <summary>
		/// returns a unqiue session name (per page) to be able to save things per page
		/// </summary>
		/// <returns></returns>
		private string _GetPageSpecificSessionName()
		{
			return "PathTrackingControl.Page." + Page.Request.Path;
		}

		/// <summary>
		/// Gets the session variable that was responsible for storing
		/// the path that came from the referrer
		/// </summary>
		/// <returns>string of the session variable name</returns>
		private string _GetRefererPageSpecificSessionName()
		{
			if (Page.Request.UrlReferrer == null)
				return "";
			return "PathTrackingControl.Page." + Page.Request.UrlReferrer.AbsolutePath;
		}

		/// <summary>
		/// Adds a new node (page) to the current tracking info
		/// </summary>
		/// <param name="pathArray"></param>
		/// <param name="node"></param>
		private void _ProcessNewPath(ref ArrayList pathArray, ref Node node)
		{
			Node currentNode;
			// search if already inside the array
			for (int i = 0; i < pathArray.Count; i++)
			{
				currentNode = (Node) pathArray[i];
				string p1 = currentNode.path;
				string p2 = node.path;

				int temp = p1.IndexOf("?");
				if (temp != -1)
					p1 = p1.Remove(temp, p1.Length - temp);

				temp = p2.IndexOf("?");
				if (temp != -1)
					p2 = p2.Remove(temp, p2.Length - temp);

				// node already in array, do not add it
				if (p1 == p2)
				{
					if (i < pathArray.Count - 1)
						pathArray.RemoveRange(i + 1, pathArray.Count - (i + 1));
					return;
				}
			}
			// node not found, a new path, so let us add it
			pathArray.Add(node);
		}
		#endregion
	}
}
